@@ -5,6 +5,8 @@
 CDMInterface <- R6::R6Class(
   classname = "CDMInterface",
   public = list(
+    ## Public ----
+    ### Methods ----
     #' @description
     #' Initializer method
     #'
@@ -16,6 +18,9 @@ CDMInterface <- R6::R6Class(
     #' @return (`invisible(self)`)
     initialize = function(connectionDetails = NULL, cdmSchema = NULL, resultSchema = NULL, cdm = NULL) {
       private$connectionDetails <- connectionDetails
+      if (!is.null(private$connectionDetails)) {
+        private$connection <- DatabaseConnector::connect(private$connectionDetails)
+      }
       private$cdmSchema <- cdmSchema
       private$resultSchema <- resultSchema
       private$cdm <- cdm
@@ -38,11 +43,40 @@ CDMInterface <- R6::R6Class(
     validate = function() {
       errorMessages <- checkmate::makeAssertCollection()
 
-      checkmate::assertClass(private$connectionDetails, "ConnectionDetails", null.ok = TRUE, add = errorMessages)
-      checkmate::assertCharacter(x = private$connectionDetails$dbms, len = 1, null.ok = TRUE, add = errorMessages)
-      checkmate::assertCharacter(private$cdmDatabaseSchema, null.ok = TRUE, len = 1, add = errorMessages)
-      checkmate::assertCharacter(private$resultSchema, null.ok = TRUE, len = 1, add = errorMessages)
-      checkmate::assertClass(private$cdm, classes = "cdm_reference", null.ok = TRUE, add = errorMessages)
+      checkmate::assertClass(
+        x = private$connectionDetails,
+        "ConnectionDetails",
+        null.ok = TRUE,
+        add = errorMessages
+      )
+      
+      checkmate::assertCharacter(
+        x = private$connectionDetails$dbms,
+        len = 1,
+        null.ok = TRUE,
+        add = errorMessages
+      )
+      
+      checkmate::assertCharacter(
+        private$cdmDatabaseSchema,
+        null.ok = TRUE,
+        len = 1,
+        add = errorMessages
+      )
+      
+      checkmate::assertCharacter(
+        private$resultSchema,
+        null.ok = TRUE,
+        len = 1,
+        add = errorMessages
+      )
+      
+      checkmate::assertClass(
+        private$cdm,
+        classes = "cdm_reference",
+        null.ok = TRUE,
+        add = errorMessages
+      )
 
       checkmate::reportAssertions(collection = errorMessages)
       return(invisible(self))
@@ -100,15 +134,32 @@ CDMInterface <- R6::R6Class(
         DatabaseConnector = private$dbconFetchMetadata(andromeda)
       )
       return(invisible(NULL))
+    },
+    
+    #' @description
+    #' Destroys instance
+    #' 
+    #' @return (NULL)
+    destroy = function() {
+      private$finalize()
     }
   ),
   private = list(
+    ## Private ----
+    ### Fields ----
     connectionDetails = NULL,
+    connection = NULL,
     cdmSchema = NULL,
     resultSchema = NULL,
     cdm = NULL,
     type = "",
 
+    ### Methods ----
+    finalize = function() {
+      DatabaseConnector::disconnect(private$connection)
+    },
+    
+    #### DatabaseConnector ----
     # cohortIds (`integer(n)`)
     # cohortTableName (`character(1)`)
     dbconFetchCohortTable = function(cohortIds, cohortTableName) {
@@ -122,14 +173,11 @@ CDMInterface <- R6::R6Class(
         cohortIds = cohortIds
       )
 
-      connection <- DatabaseConnector::connect(private$connectionDetails)
-      on.exit(DatabaseConnector::disconnect(connection))
-
       translatedSql <- SqlRender::translate(
         sql = renderedSql,
-        targetDialect = connection@dbms
+        targetDialect = private$connection@dbms
       )
-      return(DatabaseConnector::querySql(connection, translatedSql))
+      return(DatabaseConnector::querySql(private$connection, translatedSql))
     },
 
     # andromeda (`Andromeda::andromeda()`)
@@ -148,15 +196,13 @@ CDMInterface <- R6::R6Class(
         personIds = personIds
       )
 
-      connection <- DatabaseConnector::connect(private$connectionDetails)
-      on.exit(DatabaseConnector::disconnect(connection))
-
       translatedSql <- SqlRender::translate(
         sql = renderedSql,
-        targetDialect = connection@dbms
+        targetDialect = private$connection@dbms
       )
 
-      andromeda$year_of_birth <- DatabaseConnector::querySql(connection, translatedSql)
+      andromeda$year_of_birth <- DatabaseConnector::querySql(
+        private$connection, translatedSql)
 
       andromeda$treatmentHistory <- andromeda$treatmentHistory %>%
         dplyr::inner_join(andromeda$year_of_birth, by = dplyr::join_by(person_id == PERSON_ID)) %>%
@@ -184,15 +230,13 @@ CDMInterface <- R6::R6Class(
         personIds = personIds
       )
 
-      connection <- DatabaseConnector::connect(private$connectionDetails)
-      on.exit(DatabaseConnector::disconnect(connection))
-
       translatedSql <- SqlRender::translate(
         sql = renderedSql,
-        targetDialect = connection@dbms
+        targetDialect = private$connection@dbms
       )
 
-      andromeda$sex <- DatabaseConnector::querySql(connection, translatedSql)
+      andromeda$sex <- DatabaseConnector::querySql(
+        private$connection, translatedSql)
 
       andromeda$treatmentHistory <- andromeda$treatmentHistory %>%
         dplyr::inner_join(andromeda$sex, by = dplyr::join_by(person_id == PERSON_ID)) %>%
@@ -212,15 +256,13 @@ CDMInterface <- R6::R6Class(
         cdmSchema = private$cdmSchema
       )
 
-      connection <- DatabaseConnector::connect(private$connectionDetails)
-      on.exit(DatabaseConnector::disconnect(connection))
-
       translatedSql <- SqlRender::translate(
         sql = renderedSql,
-        targetDialect = connection@dbms
+        targetDialect = private$connection@dbms
       )
 
-      andromeda$metadata <- DatabaseConnector::querySql(connection, translatedSql) %>%
+      andromeda$metadata <- DatabaseConnector::querySql(
+        private$connection, translatedSql) %>%
         dplyr::mutate(
           execution_start_date = as.character(Sys.Date()),
           package_version = as.character(utils::packageVersion("TreatmentPatterns")),
@@ -228,7 +270,7 @@ CDMInterface <- R6::R6Class(
           platform = base::version$platform
         )
     },
-
+    #### CDMConnector ----
     # cohortIds (`integer(n)`)
     # cohortTableName (`character(1)`)
     cdmconFetchCohortTable = function(cohortIds, cohortTableName) {
